%t_test
% Function which performs a t-test on a set of samples. The samples are
% assumed to derive from a Normal distribution with mean mu and standard
% deviation sigma. The t-test determines whether the mean of the samples,
% which is an unbiased estimator of the true population mean, is suitably 
% close to the true mean. By 'suitably close' we state that the true mean
% is within the sample mean +/- confidence intervals, calculated to a
% specified % significance. 
%
% This function will also perform the same analysis with a normal
% distribution. (Which is the limit of the t-distribution when the number
% of samples is large, i.e. over about thirty).
%
% The t-test is named after William "Student" Sealy Gosset (1876-1937), 
% of Winchester College, who developed the test while working for 
% Guinness breweries.

% LAST UPDATED by Dr Andrew French September 2017

function t_test

%Clear screen first
clc; clear all;

%% INPUTS %%

%Define confidence level for t-test and 'g-test'
confidence_level = 0.95;

%Define test data parameters (from Normal distribution)
mu = 42;
sigma = 5;
n = 10;

%Set graph limits for t-test and normal distribution plots
tmin = -4;
tmax = 4;

%Number of plot points
N = 1000;

%Fontsize
fsize = 10;

%

%Generate test data, sampled from a Normal distribution
z = randn(1,n);
x = sigma*z + mu;

%Display the test data to the command window
disp(' ')
disp(' t-test demonstration by Dr Andrew French September 2017')
disp(' ')
disp([' Significance level = ',num2str(confidence_level*100),'%, n = ',num2str(n),' samples.'])
for k=1:n
    disp([' ',num2str(x(k))]);
end
disp(' ');

%Perform t-test (assume two-tail, i.e. mean is within symmetric confidence
%limits)
[mu_estimate, sigma_estimate, t_confidence_limit,t_cl,t_t] = ttest( x, mu, confidence_level );
disp([' Sample mean = unbiased population mean estimate =  ',num2str(mu_estimate)]);
disp([' Unbiased standard deviation estimate = ',num2str(sigma_estimate)]);
disp(' ')
disp(' t-test confidence limit is:')
disp([' ',num2str( mu_estimate - t_confidence_limit),' < = population mean <= ',num2str( mu_estimate + t_confidence_limit)]);
if (mu >= mu_estimate - t_confidence_limit) && (mu <= mu_estimate + t_confidence_limit)
    disp(' t-test PASSED')
else
    disp(' t-test FAILED')
end
disp(' ');

%Perform Norman test ("g-test")
[mu_estimate, sigma_estimate, g_confidence_limit,z_cl,z_g] = gtest( x, mu, confidence_level );
disp(' Gaussian test confidence limit is:')
disp([' ',num2str( mu_estimate - g_confidence_limit),' < = population mean <= ',num2str( mu_estimate + g_confidence_limit)]);
if (mu >= mu_estimate - g_confidence_limit) && (mu <= mu_estimate + g_confidence_limit)
    disp(' g-test PASSED')
else
    disp(' g-test FAILED')
end
disp(' ');

%

%%

%Generate a pictorial representation of the t-test 
fig = figure( 'color',[1 1 1],'name','t-test' );

%Generate PDFs for t-distribution and normal distribution
t = linspace( tmin, tmax, N );
PDF_t = tpdf(t,n-1);
PDF_g = (1/sqrt(2*pi))*exp(-0.5*t.^2);

%Plot probability density function (PDF) for t-test
subplot(2,1,1);
plot( t, PDF_t,'b','linewidth',2);

%Set grid and axes limits
hold on;
grid on;
axis tight
xlim( [tmin,tmax] )
ylim( [0, max(PDF_g) ] );
set( gca,'fontsize',fsize )
box on;

%Define patch for cumulative region
tt = linspace( -t_cl,t_cl,N );
patch( [-t_cl,tt,t_cl,-t_cl],...
    [0,tpdf(tt,n-1),0,0], 'b','facealpha',0.5);

%Plot t value for data
plot( [t_t,t_t],[0,tpdf(t_t,n-1)],'k-' );
plot( t_t, 0, 'ko','markersize',fsize );

%Check probability is what it should be between confidence interval
Pcheck = tcdf(t_cl,n-1) - tcdf(-t_cl,n-1);

%Set graph labelling
xlabel('t','fontsize',fsize);
ylabel('Probability density','fontsize',fsize)
title({['t-test for population with mean=',num2str(mu),', STD=',num2str(sigma),...
    ', mean estimate=',num2str(mu_estimate),', STD estimate=',num2str(sigma_estimate)],...
    ['P( ',num2str(-t_cl),' \leq t < ',num2str(t_cl),' ) = ',num2str(Pcheck)] },'fontsize',fsize)

%

%Plot probability density function (PDF) for g-test
subplot(2,1,2);
plot( t, PDF_g,'g','linewidth',2);

%Set grid and axes limits
hold on;
grid on;
axis tight
xlim( [tmin,tmax] )
ylim( [0, max(PDF_g) ] );
set( gca,'fontsize',fsize )
box on;

%Define patch for cumulative region
zz = linspace( -z_cl,z_cl,N );
patch( [-z_cl,zz,z_cl,-z_cl],...
    [0,(1/sqrt(2*pi))*exp(-0.5*zz.^2),0,0], 'g','facealpha',0.5);

%Plot t value for data
plot( [z_g,z_g],[0,(1/sqrt(2*pi))*exp(-0.5*z_g^2)],'k-' );
plot( z_g, 0, 'ko','markersize',fsize );

%Check probability is what it should be between confidence interval
Pcheck = 0.5*( erf(z_cl/sqrt(2)) - erf(-z_cl/sqrt(2)) );

%Set graph labelling
xlabel('z','fontsize',fsize);
ylabel('Probability density','fontsize',fsize)
title({['g-test for population with mean=',num2str(mu),', STD=',num2str(sigma),...
    ', mean estimate=',num2str(mu_estimate),', STD estimate=',num2str(sigma_estimate)],...
    ['P( ',num2str(-z_cl),' \leq z < ',num2str(z_cl),' ) = ',num2str(Pcheck)] },'fontsize',fsize)

%Save PNG file
print(fig,'t-test.png','-dpng','-r300');
close(fig);

%%

%Student's t-test
function [mu_estimate, sigma_estimate, confidence_limit,t_cl,t] = ttest( x, mu, confidence_level )

%Estimate of the population mean
mu_estimate = mean(x);

%Unbiased estimate of the population standard deviation
n = numel(x);
sigma_estimate = sqrt( (n/(n-1)) * ( mean(x.^2) - mean(x)^2 ) );

%t value for data
t = (mu - mu_estimate)/sqrt( (sigma_estimate^2) / n );

%Find t value given confidence level
t_cl = tinv( confidence_level + (1-confidence_level)/2, n - 1 );

%Determine confidence limit
confidence_limit = t_cl*sigma_estimate/sqrt(n);

%%

%Normal (Gaussian) 'g-test'
function [mu_estimate, sigma_estimate, confidence_limit, z_cl, z] = gtest( x, mu, confidence_level )

%Estimate of the population mean
mu_estimate = mean(x);

%Unbiased estimate of the population standard deviation
n = numel(x);
sigma_estimate = sqrt( (n/(n-1)) * ( mean(x.^2) - mean(x)^2 ) );

%z value for data
z = (mu - mu_estimate)/sqrt( (sigma_estimate^2) / n );

%Find z value given confidence level
P = confidence_level + (1-confidence_level)/2;
z_cl = sqrt(2)*erfinv(2*P-1);

%Determine confidence limit
confidence_limit = z_cl*sigma_estimate/sqrt(n);

%End of code